# Copyright (c) 2002-2003  Jim Bublitz <jbublitz@nwinternet.com>

# This software is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this library; see the file COPYING.
# If not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys
import re
import os
import os.path
import string

# Release numbers - don't forget to update these !!!

Project    = "PyKDE"
Release    = "3.8"
Version    = "3.8-1"

shortOptions = "hc:d:e:g:i:j:k:m:n:o:p:q:r:s:t:u:v:z:"
usingSip     = True
usingQt      = True
usingKDE     = True
usingPyQt    = True
usingPyKDE   = False
isPyKDE      = True
explicitMake = False
catCppFiles  = True
libDir       = "libs"

platCopy     = None
platMove     = None
platRemove   = None
platMkdir    = None
platRmdir    = None

platformTag  = "WS_X11"

# QtVers must match qttags
QtVers     = ["300", "301", "302", "304", "305", "306", "310", "311", "312", "320"]
KDEVers    = ["300", "301", "303", "310", "311", "312", "313", "314"]

manifest   = ["pythonize", "pythonize/test", "dcop", "kdecore", "kdesu", "kdefx",
              "kdeui", "kio", "kfile", "kparts", "khtml", "kjs", "kspell",
              "kdeprint", "pykpanelapplet"]

# module: (bldtype, postProc)
modInfo    = {"pythonize":      ("clib", None),
              "pythonize/test": ("clib", None),
              "dcop":           ("siplib", None),
              "kdecore":        ("siplib", ["-p ", "kdecore", "-o", "appQuit",  "kdecore.py"]),
              "kdesu":          ("siplib", None),
              "kdefx":          ("siplib", None),
              "kdeui":          ("siplib", ["-p ", "kdeui", "-o", "shpix",  "sipkdeuiKSharedPixmap.cpp"]),
              "kio":            ("siplib", None),
              "kfile":          ("siplib", None),
              "kparts":         ("siplib", None),
              "khtml":          ("siplib", None),
              "kjs":            ("siplib", None),
              "kspell":         ("siplib", None),
              "kdeprint":       ("siplib", None),
              "pykpanelapplet": ("clib", None)
             }

# Want this global so it's testable by external program
qttags = {0x020000: "Qt_1_43",
          0x020100: "Qt_2_00",
          0x020200: "Qt_2_1_0",
          0x020300: "Qt_2_2_0",
          0x020301: "Qt_2_3_0",
          0x030000: "Qt_2_3_1",
          0x030001: "Qt_3_0_0",
          0x030002: "Qt_3_0_1",
          0x030004: "Qt_3_0_2",
          0x030005: "Qt_3_0_4",
          0x030006: "Qt_3_0_5",
          0x030100: "Qt_3_0_6",
          0x030101: "Qt_3_1_0",
          0x030102: "Qt_3_1_1",
          0x030200: "Qt_3_1_2",
          0x040000: "Qt_3_2_0"
        }


usageTxt = \
"""
%s %s build script

Usage:
    %s [-h]
    %s [options]

where:
    -h             display this help message

options:
    -c+            concatenate each module's C++ source files [default: use -c- to disable]
    -d dir         where PyKDE will be installed
    -e dir         the directory containing the SIP header files
    -g prog        the complete path/name of the Makefile generator"
    -i dir         the directory containing the Qt header files
    -j #           split the concatenated C++ source files into # pieces
    -k dir         the KDE base directory
    -m prog        the name of the Make program
    -n dir         the directory containing the KDE header files
    -o dir         the directory containing the KDE lib files
    -p dir         the name of the SIP code generator [default sip]
    -q dir         the Qt base directory
    -r dir         the directory containing the Qt library
    -s dir         the directory containing the SIP module
    -t dir         the directory containing the Python library directories
    -u value       version checking on for PyQt and sip [all}, off for sip [nosip], or none [none] (default all)
    -v dir         the directory containing the PyQt sip files
    -z file        the name of a file containing command line flags
"""

def configError (msg):
    print "\nError: %s\n" % msg
    sys.exit (1)

def addOptions (optlist, args):
    pass

def generateHook (dis):
    cwd = os.getcwd ()
    wabbitExeDir     = os.path.join (cwd, "doc", "wabbit")
    wabbitInstallDir = os.path.join (cwd, "doc", "wabbit", "install")
    m = open ("doc/wabbit/install/wabbitrc.in", "r")
    n = open ("doc/wabbit/install/wabbitrc", "w")

    buff = m.read ()
    m.close ()
    buff = string.replace (buff, "@PYKDEREF@", os.path.join (cwd, "doc", "wabbit", "data"))
    buff = string.replace (buff, "@PYKDEDOC@", os.path.join (cwd, "doc"))
    n.write (buff)
    n.close ()

def addCLibProPatches (dis, patches):
    pass

def addCLibMakefilePatches (dis, patches):
    pass

def customCLibInstall (dis, mname, installTxt):
    # returns installTxt, cleanTxt, lnSrc, lnDst - see build.py
    cwd = os.getcwd ()
    statObj = os.stat (dis.home)
    uid = statObj.st_uid
    gid = statObj.st_gid
    wabbitExeDir     = os.path.join (cwd, "doc", "wabbit")
    wabbitInstallDir = os.path.join (cwd, "doc", "wabbit", "install")
    wabbitDir = os.path.join (dis.home, ".wabbit")
    src       = os.path.join (wabbitInstallDir, "*")
    installTxt = installTxt + "\t-mkdir -p %s\n" % (wabbitDir)\
                 + "\t-chown %i:%i %s\n" % (uid, gid, wabbitDir)\
                 + "\t-cp %s %s\n" % (src, wabbitDir)\
                 + "\t-chown %i:%i %s\n" % (uid, gid, os.path.join (wabbitDir, "*"))\
                 + "\t-ln -s %s %s\n" % (os.path.join (wabbitExeDir, "wabbit.py"), os.path.join (os.sep, "usr", "bin", "wabbit"))

    return installTxt, None, None, None

def addSipLibProPatches (dis, patches):
    pass

def addSipLibMakefilePatches (dis, patches):
    pass

def customSipLibInstall (dis, mname, installTxt, cleanTxt):
    return installTxt, cleanTxt

