"""
This is the installer for KPanelApplets
"""

"""
Copyright 2003 Jim Bublitz <jbublitz@nwinternet.com>

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; see the file COPYING
If not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""


False    = 0
True     = not False
nobody   = 65534
root     = 0
nochange = -1


import sys, os, os.path, string, shutil, imp, types, py_compile, re

from qt import Qt, QPopupMenu, SIGNAL, QWidget, QLabel, QCheckBox, QPushButton, QString, QCloseEvent,\
               QRadioButton, QButtonGroup, QValidator

from kdecore import KApplication, KCmdLineArgs, KAboutData, i18n, KShortcut, KGlobal, KStandardDirs, KURL,\
                    KConfig
from kdeui import KMainWindow, KMessageBox, KStdAction, KAction, KTabCtl, KLineEdit, KSeparator, KDialog,\
                  KIntValidator
from kfile import KURLRequester

STATUSBAR_LEFT   = 1
STATUSBAR_MIDDLE = 2
STATUSBAR_RIGHT  = 3

class MainWin (KMainWindow):
    def __init__ (self, *args):
        apply (KMainWindow.__init__, (self,) + args)

        self.setCaption ("Python KPanelApplet Installer")
        self.initActions ()
        self.initMenus ()
        self.initStatusBar ()
        self.handler = AppletHandler (self)
        self.setGeometry (100, 100, 500, 400)

        self.tabCtl = KTabCtl (self)
        self.tabCtl.setGeometry (10, 40, 480, 330)
        self.connect (self.tabCtl, SIGNAL ("tabSelected (int)"), self.slotTabSelected)

        self.installPage = Page1 (self.tabCtl, self.handler)
        self.uninstallPage = Page2 (self.tabCtl, self.handler)
        self.tabCtl.addTab (self.installPage, "Install")
        self.tabCtl.addTab (self.uninstallPage, "Uninstall")
        self.tabs = [self.installPage, self.uninstallPage]

        self.checkRoot ()

    def initActions (self):
        # "File" menu items
        self.quitAction   = KStdAction.quit (self.slotQuit, self.actionCollection ())


    def initMenus (self):
        fileMenu = QPopupMenu (self)
        self.quitAction.plug (fileMenu)
        self.menuBar ().insertItem (i18n ("&File"), fileMenu)

        helpMenu = self.helpMenu ("")
        self.menuBar ().insertItem (i18n ("&Help"), helpMenu)

    def initStatusBar (self):
        self.statusBar ().insertItem ("", STATUSBAR_LEFT, 1000, True)
        self.statusBar ().insertItem ("", STATUSBAR_MIDDLE, 1000, True)
        self.statusBar ().insertItem ("", STATUSBAR_RIGHT, 1000, True)

    def checkRoot (self):
        if os.getuid () != 0:
            KMessageBox.error (self,"\nYou must be root to run this installer\n\n", "Panel Applet Installer: Got root?")
            sys.exit (1)

    def queryExit (self):
        return True

    def queryClose (self):
        return True


#-------------------- slots -----------------------------------------------

    def slotQuit (self):
        KApplication.kApplication ().quit ()

    def slotTabSelected (self, t):
        self.tabs [t - 1].scriptName.lineEdit ().setText ("")
        self.tabs [t-1].slotTextChanged ("")

    def notImpl (self, item = "Feature"):
        self.statusBar ().changeItem ("%s not implemented" % item, STATUSBAR_LEFT)
        KMessageBox.error (self, "%s not implemented" % item, "Not Implemented")
        self.statusBar ().changeItem ("", STATUSBAR_LEFT)

#-------------------- Classes ---------------------------------------------

class Page1 (QWidget):
    def __init__ (self, parent, handler, name = ""):
        QWidget.__init__ (self, parent, name)

        self.handler = handler

        snLbl = QLabel ("Script for panel applet", self)
        snLbl.setGeometry (75, 15, 250, 20)
        self.scriptName = KURLRequester (self)
        self.scriptName.setGeometry (75, 35, 300, 25)
        self.scriptName.setFilter ("*.py")
        self.connect (self.scriptName, SIGNAL ("textChanged (const QString&)"), self.slotTextChanged)

        self.replaceCB = QCheckBox ("Replace existing applet", self)
        self.replaceCB.setGeometry (125, 80, 175, 20)
        self.replaceCB.setChecked (False)
        self.replaceCB.setEnabled (False)
        self.connect (self.replaceCB, SIGNAL ("toggled (bool)"), self.slotReplaceToggled)

        self.testPB = QPushButton ("Test", self)
        self.testPB.setGeometry (100, 125, 70, 22)
        self.connect (self.testPB, SIGNAL ("clicked ()"), self.slotTest)
        self.testPB.setEnabled (False)

        self.installPB = QPushButton ("Install", self)
        self.installPB.setGeometry (175, 125, 75, 22)
        self.connect (self.installPB, SIGNAL ("clicked ()"), self.slotInstall)
        self.installPB.setEnabled (False)

        self.quitPB = QPushButton ("Quit", self)
        self.quitPB.setGeometry (255, 125, 70, 22)
        self.connect (self.quitPB, SIGNAL ("clicked ()"), self.slotQuit)

        dtLbl = QLabel ("Desktop file", self)
        dtLbl.setGeometry (40, 180, 75, 20)

        sep = KSeparator (self)
        sep.setGeometry (140, 190, 250, 5)

        self.nameLbl  = QLabel ("Menu name", self)
        self.nameLbl.setGeometry (75, 210, 75, 20)
        self.nameEdit = KLineEdit (self)
        self.nameEdit.setGeometry (155, 210, 200, 20)
        self.nameEdit.setEnabled (False)
        self.connect (self.nameEdit, SIGNAL ("textChanged (const QString&)"), self.slotNameTextChanged)

        self.cmtLbl  = QLabel ("Comment", self)
        self.cmtLbl.setGeometry (75, 240, 75, 20)
        self.cmtEdit = KLineEdit (self)
        self.cmtEdit.setGeometry (155, 240, 200, 20)
        self.cmtEdit.setEnabled (False)
        self.connect (self.cmtEdit, SIGNAL ("textChanged (const QString&)"), self.slotCmtTextChanged)

        self.uniqueCB = QCheckBox ("Unique Applet", self)
        self.uniqueCB.setGeometry (75, 270, 125, 20)
        self.uniqueCB.setChecked (False)
        self.uniqueCB.setEnabled (False)
        self.connect (self.uniqueCB, SIGNAL ("toggled (bool)"), self.slotUniqueToggled)

        self.validator = AppletValidator (self)
        self.tester    = AppletTester (self)

    #-------------------- slots -------------------------------------------

    def enabled (self, state):
        widList = [self.installPB, self.testPB, self.nameEdit, self.cmtEdit, self.uniqueCB]
        for widget in widList:
            widget.setEnabled (state)

    def slotTextChanged (self, s):
        if os.access (str (s), os.F_OK) and self.validator.validate (str (s)):
            isInstalled = self.handler.isAppletInstalled (str (s))
            self.replaceCB.setChecked (isInstalled)
            self.replaceCB.setEnabled (isInstalled)
            self.enabled (True)
            self.nameEdit.setText (self.handler.menuName)
            self.cmtEdit.setText (self.handler.comment)
            self.uniqueCB.setEnabled (self.handler.unique)

        else:
            self.replaceCB.setChecked (False)
            self.replaceCB.setEnabled (False)
            self.enabled (False)
            self.nameEdit.setText (self.handler.menuName)
            self.cmtEdit.setText (self.handler.comment)
            self.uniqueCB.setEnabled (self.handler.unique)

    def slotInstall (self):
        self.handler.installApplet (str (self.scriptName.lineEdit ().text ()))

    def slotTest (self):
        test = self.tester.runTest (str (self.scriptName.lineEdit ().text ()))

    def slotQuit (self):
        KApplication.kApplication ().quit ()

    def slotNameTextChanged (self, s):
        self.handler.menuName = str (s)

    def slotCmtTextChanged (self, s):
        self.handler.comment = str (s)

    def slotUniqueToggled (self, checked):
        self.handler.unique = checked

    def slotReplaceToggled (self, checked):
        self.handler.replace = checked
        if not checked\
            and self.installPB.isEnabled ()\
            and self.handler.isAppletInstalled (str (self.scriptName.lineEdit ().text ())):
            self.installPB.setEnabled (False)
        elif checked and self.testPB.isEnabled ():
            self.installPB.setEnabled (True)

class Page2 (QWidget):
    def __init__ (self, parent, handler, name = ""):
        QWidget.__init__ (self, parent, name)

        self.handler = handler

        snLbl = QLabel ("Script for panel applet", self)
        snLbl.setGeometry (75, 75, 250, 20)
        self.scriptName = KURLRequester (self)
        self.scriptName.setGeometry (75, 95, 300, 25)
        self.scriptName.setFilter ("*.py")
        self.scriptName.fileDialog ().setURL (KURL (self.handler.appletPath ()))
        self.connect (self.scriptName, SIGNAL ("textChanged (const QString&)"), self.slotTextChanged)

        self.uninstallPB = QPushButton ("Uninstall", self)
        self.uninstallPB.setGeometry (135, 200, 75, 22)
        self.connect (self.uninstallPB, SIGNAL ("clicked ()"), self.slotUninstall)
        self.uninstallPB.setEnabled (False)

        self.quitPB = QPushButton ("Quit", self)
        self.quitPB.setGeometry (215, 200, 70, 22)
        self.connect (self.quitPB, SIGNAL ("clicked ()"), self.slotQuit)

    #-------------------- slots -------------------------------------------

    def slotTextChanged (self, s):
        self.uninstallPB.setEnabled (self.handler.isAppletInstalled (self.handler.createBaseName (str (s))))

    def slotUninstall (self):
        self.handler.uninstallApplet (self.handler.createBaseName (str (self.scriptName.lineEdit (). text ())), True)

    def slotQuit (self):
        KApplication.kApplication ().quit ()


laTemplate = """# %s.la - a fake libtool library file
# Generated by installPanelApplet
#
# Please DO NOT delete this file!
# It is necessary for linking the library.

# The name that we can dlopen(3).
dlname='%s.so'

# Names of this library.
library_names='%s.so %s.so %s.so'

# The name of the static archive.
old_library=''

# Libraries that this one depends upon.
dependency_libs=''

# Version information for clock_panelapplet.
current=0
age=0
revision=0

# Is this an already installed library?
installed=yes

# Files to dlopen/dlpreopen
dlopen=''
dlpreopen=''

# Directory that this library needs to be installed in:
libdir='%s'"""


dtContents = """[Desktop Entry]
Name=%s
Comment=%s
X-KDE-Library=%s
X-KDE-UniqueApplet=%s
"""


class AppletHandler:
    def __init__ (self, parent):
        self.parent = parent
        self. KDEDir = self.findKDEDir ()
        if self.KDEDir is None:
            self.error ("Couldn't find top level KDE directory on your system", "Couldn't find KDE directories")

        if not os.access (os.path.join (self.appletLibPath (), "libpykpanelapplet.so"), os.F_OK)\
            or not os.access (os.path.join (self.appletLibPath (), "libpykpanelapplet.so"), os.F_OK):
            self.error ("%s/libpykpanelapplet.so or .la (or both) missing" % self.appletLibPath())

        self.replace  = False
        self.hasDTF   = False

    def clearFields (self):
        self.menuName = ""
        self.comment  = ""
        self.unique   = False

    def error (self, msg, caption = "Panel Applet Installer Error"):
        KMessageBox.error (self.parent, msg, caption)
        raise
        sys.exit (1)


    def isAppletInstalled (self, pyName, readDesktop = True):
        name = self.createAppletName (pyName)
        haveApplet = name != "" and  os.access (os.path.join (self.appletPath (), name + ".py"), os.F_OK)
        if not haveApplet:
            self.clearFields ()
        elif readDesktop:
            self.readDesktopFile (pyName)
        return haveApplet

    def readDesktopFile (self, pyName):
        if pyName [-3:] == ".py":
            dtName = pyName [:-3] + ".desktop"
        else:
            dtName = pyName + ".desktop"

        if not os.access (dtName, os.F_OK):
            self.clearFields ()
            return False

        cfg = KConfig (dtName, False, False)
        cfg.setGroup ("Desktop Entry")
        self.menuName = cfg.readEntry ("Name")
        self.comment = cfg.readEntry ("Comment")
#         cfg.readEntry ("X-KDE-Library")
        self.unique = str(cfg.readEntry ("X-KDE-UniqueApplet")).lower () == "true"

        return True

    def updateDesktopFile (self, pyName):
        if pyName [-3:] == ".py":
            dtName = pyName [:-3] + ".desktop"
        else:
            dtName = pyName + ".desktop"

        if not os.access (dtName, os.F_OK):
            return None

        cfg = KConfig (dtName, False, False)
        cfg.setGroup ("Desktop Entry")
        cfg.writeEntry ("Name", self.menuName)
        cfg.writeEntry ("Comment", self.comment)
        if self.unique:
            cfg.writeEntry ("X-KDE-UniqueApplet", "true")
        else:
            cfg.writeEntry ("X-KDE-UniqueApplet", "false")

        cfg.writeEntry ("X-KDE-Library", self.createAppletName (pyName))
        return dtName

    def installApplet (self, pyName):
        if self.isAppletInstalled (pyName, False) and self.replace:
            self.uninstallApplet (pyName)

        self.createAppletLib (self.createAppletName (pyName))
        self.fakeLA (self.createAppletName (pyName))
        self.writeAppletDesktop (pyName)
        if pyName [-3:] != ".py": pyName += ".py"
        shutil.copyfile (pyName, os.path.join (self.appletPath (), self.createAppletName (pyName, ".py")))
        KMessageBox.information (self.parent, "Installation was successful\nPanel Applet is ready to use", "Panel Applet Installer")

    def uninstallApplet (self, pyName, showMsg = False):
        appletName = self.createAppletName (pyName)
        appPath    = os.path.join (self.appletPath (), appletName)
        appLibPath = os.path.join (self.appletLibPath (), appletName)
        targets = [appPath + ".desktop", appPath + ".py", appPath + ".pyc",\
                   appLibPath + ".so", appLibPath + ".la"]

        for target in targets:
            try:
                if os.access (target, os.F_OK):
                    os.unlink (target)
            except:
                self.error ("Uninstall failed -- %s" % target)

        if showMsg:
            KMessageBox.information (self.parent, "Uninstall was successful", "Panel Applet Installer")


    def createBaseName (self, instName):
        pos = instName.find ("_py_applet")
        if pos > 0:
            return instName [:pos]
        else:
            return instName


    def createAppletName (self, pyName, ext = ""):
        try:
            return os.path.splitext (os.path.basename (pyName))[0] + "_py_applet" + ext
        except:
            return ""

    def createAppletLib (self, appletName):
        target = os.path.join (self.appletLibPath (), appletName + ".so")
        try:
            # have to unlink to relink here, regardless of self.replace
            if os.access (target, os.F_OK):
                os.unlink (target)
            os.symlink (os.path.join (self.appletLibPath (), "libpykpanelapplet.so"), target)
        except:
            self.error ("symlink to libpykpanelapplet.so failed")

    def writeAppletDesktop (self, pyName):
        appletName = self.createAppletName (pyName)
        dtFile = os.path.join (self.appletPath (), appletName + ".desktop")

        existingDTF = self.updateDesktopFile (pyName)
        if existingDTF:
            shutil.copyfile (existingDTF, dtFile)
            return


        try:
            f = open (dtFile, "w")
        except:
            self.error ("Couldn't open file %s" % dtFile)

        if not self.menuName:
            self.menuName = string.capitalize (appletName [:-10])

        if not self.comment:
            self.comment = string.capitalize (appletName [:-10]) + " - Python"

        if self.unique:
            u = "true"
        else:
            u = "false"

        f.write (dtContents % (self.menuName, self.comment, appletName, u))
        f.close ()

    def fakeLA (self, appletName):
        try:
            f = open (os.path.join (self.appletLibPath (), appletName + ".la"), "w")
        except:
            self.error ("Couldn't create %s" % os.path.join (self.appletLibPath (), appletName + ".la"))

        f.write (laTemplate % (appletName, appletName, appletName, appletName, appletName, self.appletLibPath ()))
        f.close ()

    def appletPath (self):
        return os.path.join (self.KDEDir, KStandardDirs.kde_default ("data").latin1 (), "kicker", "applets")

    def appletLibPath (self):
        return os.path.join (self.KDEDir, KStandardDirs.kde_default ("lib").latin1 (), "kde3")

    def findKDEDir (self):
        kdeDir = None
        kdeIncDir = None

        try:
            kdeDir = os.environ["KDEDIR"]
            kdeIncDir = self.checkKDEIncl (os.path.join (kdeDir, "include"))
            if not kdeIncDir:
                kdeIncDir = self.checkKDEIncl (os.path.join (kdeDir, "include", "kde"))

        except KeyError:
            pass

        if kdeDir is None and kdeIncDir is None:
            inclCandidates = ["/opt/kde3/include", "/usr/include/kde", "/usr/include"]
            for path in inclCandidates:
                if self.checkKDEIncl (path) != "":
                    kdeIncDir = path
                    break

        if kdeDir is None and kdeIncDir:
            last = string.find (kdeIncDir, "/include")
            if last > 0:
                kdeDir = kdeIncDir [:last]
                os.environ["KDEDIR"] = kdeDir

        return kdeDir

    def checkKDEIncl (self, path):
        if os.access (os.path.join (path, "kapplication.h"), os.F_OK):
            return path
        else:
            return ""

class AppletValidator:
    def __init__ (self, parent):
        self.parent = parent

    def validate (self, pyName):
        createApp_re = re.compile ("def\screateApplet.*\(\w.*\,.*\w.*\)\s*\:")
        try:
            py_compile.compile (pyName)
        except:
            pass

        os.setreuid (nochange, nobody)

        if not os.access (pyName, os.F_OK):
            KMessageBox.error (self.parent, "Applet not found %s" % pyName, "Panel Applet Installer: validate")

        path = os.path.split (pyName) [0]
        name = os.path.splitext (os.path.basename (pyName)) [0]

        try:
            f, pathname, descrip = imp.find_module (name, [path])
        except:
            KMessageBox.error (self.parent, "Unable to find module %s" % pyName, "Panel Applet Installer: validate")
            return False

        buff = f.read ()
        if not createApp_re.search (buff):
            KMessageBox.error (self.parent, "Not a panel applet %s" % pyName, "Panel Applet Installer: validate")
            f.close ()
            return False

        try:
            mod = imp.load_module (name, f, pathname, descrip)
        except:
            KMessageBox.error (self.parent, "Unable to load module -- check for errors", "Panel Applet Installer: validate")
            return False

        if not vars (mod).has_key ("createApplet"):
            KMessageBox.error (self.parent, "Module failed to load or missing 'createApplet' factory function", "Panel Applet Installer: validate")
            return False

        try:
            self.applet = vars (mod)["createApplet"] (self.parent, None)
        except:
            KMessageBox.error (self.parent, "Factory function failed", "Panel Applet Installer: validate")
            raise
            return False

        if not self.applet:
            KMessageBox.error (self.parent, "Failed to construct applet", "Panel Applet Installer: validate")
            return False

        if not self.applet.inherits ("PyKPanelApplet"):
            KMessageBox.error (self.parent, "Applet not a PyKPanelApplet subclass", "Panel Applet Installer: validate")
            return False

        del self.applet
        os.setreuid (nochange, root)
        return True


class AppletTester:
    def __init__ (self, parent):
        self.parent = parent
        self.size = 0
        self.orientation = 0

    def checkType (self, meth):
        try:
            return type (meth) == types.MethodType
        except:
            return False

    def createSizeDlg (self):
        self.sizeDlg = KDialog (self.parent)

        self.sizeLbl = QLabel ("Pixels", self.sizeDlg)
        self.sizeLbl.setGeometry (25, 15, 50, 20)

        self.sizer = KLineEdit (self.sizeDlg)
        self.sizer.setGeometry (80, 15, 50, 20)
        self.sizer.setText ("40")

        sizeVal = KIntValidator (20, 150, self.sizeDlg)
        self.sizer.setValidator (sizeVal)

        grp = QButtonGroup (3, Qt.Vertical, "Orientation", self.sizeDlg)
        grp.setGeometry (25, 50, 125, 90)
        self.horiz = QRadioButton ("Horizontal", grp)
        self.vert = QRadioButton ("Vertical", grp)
        self.xy  = QRadioButton ("Ignore", grp)

        self.horiz.setEnabled (self.checkType (self.applet.widthForHeight))
        self.vert.setEnabled (self.checkType (self.applet.heightForWidth))
        self.xy.setEnabled (not (self.horiz.isEnabled () or self.vert.isEnabled ()))

        if not (self.horiz.isEnabled () or self.vert.isEnabled ()):
            self.xy.setChecked (True)
        elif self.horiz.isEnabled ():
            self.horiz.setChecked (True)
        elif self.vert.isEnabled ():
            self.vert.setChecked (True)

        okBtn = QPushButton ("OK", self.sizeDlg)
        okBtn.setGeometry (15, 160, 65, 25)
        okBtn.setDefault (False)
        self.sizeDlg.connect (okBtn, SIGNAL ("clicked ()"), self.slotOK)

        cancelBtn = QPushButton ("Cancel", self.sizeDlg)
        cancelBtn.setGeometry (90, 160, 65, 25)
        self.sizeDlg.connect (cancelBtn, SIGNAL ("clicked ()"), self.slotCancel)


    def slotOK (self):
        if str (self.sizer.text ()) == "":
            self.size = 40
        else:
            self.size = self.sizer.text ().toInt () [0]

        if self.horiz.isChecked ():
            self.orientation = 0
        elif self.vert.isChecked ():
            self.orientation = 1
        else:
            self.orientation = 2

        self.sizeDlg.done (True)

    def slotCancel (self):
        self.sizeDlg.done (False)

    def runTest (self, pyName):

        os.setreuid (nochange, nobody)
        self.testDlg = KDialog (self.parent)

        path = os.path.split (pyName) [0]
        name = os.path.splitext (os.path.basename (pyName)) [0]

        f, pathname, descrip = imp.find_module (name, [path])
        mod = imp.load_module (name, f, pathname, descrip)

        self.applet = vars (mod)["createApplet"] (self.testDlg, None)

        self.createSizeDlg ()
        if not self.sizeDlg.exec_loop ():
            return

        if self.orientation == 0:
            self.applet.setFixedSize (self.size, self.applet.heightForWidth (self.size))
        elif self.orientation == 1:
            self.applet.setFixedSize (self.applet.widthForHeight (self.size), self.size)
        else:
            self.applet.setFixedSize (self.size, self.size)

        self.applet.show ()

        self.testDlg.exec_loop ()
        os.setreuid (nochange, root)


#-------------------- main ------------------------------------------------

if __name__ == "__main__":
    description = "Python KPanelApplet Installer"
    version     = "1.0"
    aboutData   = KAboutData ("", "",\
        version, description, KAboutData.License_GPL,\
        "(C) 2003 Jim Bublitz")

    aboutData.addAuthor ("Jim Bublitz", "", "jbublitz@nwinternet.com")
    aboutData.addCredit ("David Boddie", "development of PyKDE KPanelApplet support")

    KCmdLineArgs.init (sys.argv, aboutData)

    app = KApplication ()
    mainWindow = MainWin (None, "main window")
    mainWindow.show()
    app.exec_loop()
