
// Copyright 2003 David Boddie & Jim Bublitz

// This software is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; see the file COPYING.
// If not, write to the Free Software Foundation, Inc.,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <pythonize.h>
#include <qobject.h>
#include <qwidget.h>
#include <qframe.h>
#include <kpanelapplet.h>
#include <pykpanelapplet.h>
#include <kapplication.h>
#include <kglobal.h>
#include <klocale.h>
#include <ksimpleconfig.h>
#include <kstandarddirs.h>
#include <qstring.h>
#include <qregexp.h>
#include <unistd.h>

#define debug 1

extern PyKPanelApplet *sipForceConvertTo_PyKPanelApplet (PyObject *, int *);

PyObject *panelApplet;
Pythonize *pyize;

extern "C"
{
// These are in this file
KPanelApplet* init(QWidget *parent, const QString& script);
QString findScript (const QString& configFile);
KPanelApplet *pykp_error (char *msg);

PyObject *sipClass_QWidget;
PyObject *sipClass_QString;

// This really is 'extern'
PyObject *sipMapCppToSelf Py_PROTO((const void *,PyObject *));



KPanelApplet *pykp_error (char *msg)
{
    if (debug) printf ("error: %s\n", msg);
    return NULL;
}


QString findScript (const QString& configFile)
{
    QString script;
    // code is currently 20 random chars - put in some leeway
    QRegExp re_random ("_(.{15,25})_rc");

    if (debug) printf ("configFile: %s\n", configFile.latin1 ());

    // does configFile end with "_" + <15 -25 random chars> + "_rc"?
    // if no, it might be a script name (if KDE has been modified)
    int pos = re_random.searchRev (configFile);

    if (pos == -1)
        script = configFile.right (2) == "rc" ? configFile.left (configFile.length () - 2) : configFile;
    else
        script = configFile.left (pos);

    if (KGlobal::dirs ()->findResource ("applets", script + ".py").isNull ())
        script = "";

    return script;
}



KPanelApplet* init(QWidget *parent, const QString& configFile)
{
    if (debug) printf ("In init\n");

    // "applets" is not a standard resource type
    KGlobal::dirs()->addResourceType("applets", KStandardDirs::kde_default("data") +
                                     "kicker/applets");

    // Try to determine what py script we're loading
    QString script = findScript (configFile);
    if (script == "") return pykp_error ("***Failed to find script");

    // start the interpreter
    pyize = new Pythonize ();
    if (!pyize || !pyize->getPythonInit ()) return pykp_error ("***Failed to start interpreter\n");

    // locate the path to the script and append it to sys.path
    QString path = KGlobal::dirs ()->findResourceDir ("applets", script + ".py");
    if (path == QString::null) return pykp_error ("***Failed to locate script path");

    if (!pyize->appendToSysPath (path.latin1 ())) return pykp_error ("***Failed to set sys.path\n");

    // load the py script
    PyObject *appletModule = pyize->importModule ((char *)script.latin1 ());
    if (!appletModule) return pykp_error ("***failed to import applet module\n");

    // get the py factory function
    PyObject *appletFactory = pyize->getNewObjectRef (appletModule, "createApplet");
    if (!appletFactory) return pykp_error ("***failed to find applet factory\n");

    // set up the args for the python factory function call
    PyObject *pyConfigFile = sipMapCppToSelf (&configFile, sipClass_QString);
    PyObject *pyParent     = sipMapCppToSelf (parent, sipClass_QWidget);
    PyObject *args         = Py_BuildValue ("NN", pyParent, pyConfigFile);

    if (pyConfigFile && pyParent && args)
    {
        // run the factory function
        panelApplet = pyize->runFunction (appletFactory, args);
        if (!panelApplet) pykp_error ("*** runFunction failure\n;");
    }
    else if (debug)
        pykp_error ("***failed to create args\n");

    // cleanup
    pyize->decref (pyConfigFile);
    pyize->decref (pyParent);
    pyize->decref (args);
    pyize->decref (appletFactory);

    PyKPanelApplet *applet;

    // convert the PyObject to a C++ PyKPanelApplet *
    int isErr = 0;
    applet = sipForceConvertTo_PyKPanelApplet (panelApplet, &isErr);
    if (isErr) return pykp_error ("***failed sip conversion to C++ pointer\n");

    // PyKDE can't run the applet without this - Pythonize
    // grabs the lock at initialization and we have to give
    // it back before exiting. At this point, we no longer need
    // it.
    pyize->releaseLock ();

//    applet = new PyKPanelApplet (configFile, KPanelApplet::Normal, 0, parent, "demo");

    // set a pointer in the applet so the interpreter will
    // be shut down when the applet exits
    applet->setPythonize (pyize);

    // take care of any translation info
    KGlobal::locale()->insertCatalogue(applet->name ());

     // pass a pointer to the applet back to kicker
    return applet;
}

}



/*PyKPanelApplet::PyKPanelApplet (const QString& configFile, Type t, int actions,
                                       QWidget *parent, const char *name,
                                       WFlags f) : KPanelApplet (configFile, t, actions, parent, name, f)
{
    pyize = NULL;
}

PyKPanelApplet::~PyKPanelApplet ()
{
    // shuts down the interpreter
    printf ("PyKPanelApplet destructor called\n");
    Py_XDECREF (panelApplet);
    if (pyize) delete pyize;
};*/





/*CPanelApplet::CPanelApplet (const QString& configFile, Type t,
                            int actions, QWidget *parent,
                                       const char *name,
                                       WFlags f) : KPanelApplet (configFile, t, actions, parent, name, f)
{
  setPaletteBackgroundColor( QColor (0x00, 0x00, 0xff));

  setFrameStyle( StyledPanel | Sunken );
}

int CPanelApplet::widthForHeight( int h ) const
{
  return h; // we want to be quadratic
}

int CPanelApplet::heightForWidth( int w ) const
{
  return w; // we want to be quadratic
}
*/
