/* copyright 2003 Jim Bublitz <jbublitz@nwinternet.com>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/


#include <Python.h>
#include <pythonize.h>
#include <testPythonize.h>

#include <stdio.h>
#include <string.h>

#define PATH_HERE "/home/jim/pythonize2/pythonize/test"

TestPythonize::TestPythonize ()
{
    failed = 0;

    pyize  = new Pythonize ();
    testInit = pyize->getPythonInit ();
    reportResult ("pythonInit", testInit);

    if (!testInit) return;

    pyize->setMainModule ();
    reportResult ("sysModule", pyize->getSysModule () ? true : false);
    reportResult ("mainModule", pyize->getMainModule ()  ? true : false);
 }

TestPythonize::~TestPythonize ()
{
}

  // bool appendToSysPath (const char* newPath)
bool TestPythonize::testAppendToSysPath ()
{
    reportResult ("appendToSysPath - 1", pyize->appendToSysPath (PATH_HERE));
    char *pyPath = Py_GetPath ();

    PyObject *pathList = pyize->getNewObjectRef (pyize->getSysModule (), "path");
    reportResult ("appendToSysPath - 2", pathList ? true : false);

    int len = PyList_GET_SIZE (pathList);

    // PyList_GetItem returns a borrowed reference
    char *lastItem = PyString_AsString (PyList_GetItem (pathList, len - 1));
    reportResult ("appendToSysPath - 3", strcmp (lastItem, PATH_HERE) == 0);

    pyize->decref (pathList);
}


  //PyObject *importModule (char *moduleName)
bool TestPythonize::testImportModule ()
{
    impModule = pyize->importModule ("testModule");
    reportResult ("importModule",  impModule ? true : false);
}


  //PyObject getNewObjectRef (PyObject *module, char *object) { return PyObject_GetAttrString (module, object) }
bool TestPythonize::testGetNewObjectRef ()
{
    pyAdd = pyize->getNewObjectRef (impModule, "add");
    reportResult ("getNewObjectRef", pyAdd ? true : false);

    pyize->decref (pyAdd);
}


  // bool runScript (char *scriptPath)
bool TestPythonize::testRunScript ()
{
    reportResult ("runScript", pyize->runScript ("hello.py"));
}


  // bool runString (char *str)
bool TestPythonize::testRunString ()
{
    reportResult ("runString - 1", pyize->runString ("x = 5\n"));
    PyObject *pyX = pyize->getNewObjectRef (pyize->getMainModule (), "x");
    reportResult ("runString - 2", PyInt_AsLong (pyX) == 5);

    pyize->decref (pyX);
}


  // PyObject *runFunction (PyObject *object, PyObject *args)
bool TestPythonize::testRunFunction ()
{
    PyObject *addArgs = Py_BuildValue ("ii", 7, 6);
    PyObject *o = pyize->runFunction (pyAdd, addArgs);
    reportResult ("runFunction - 1", o != NULL);
    reportResult ("runFunction - 2", PyInt_Check (o));

    long result = 0;
    if (o)
        result = PyInt_AS_LONG (o);

    reportResult ("runFunction - 3", result == 13);

    decref (addArgs);
    decref (o);
}

bool TestPythonize::testDtor ()
{
    printf ("\n\n");
    delete pyize;
}

bool TestPythonize::runTests ()
{
    printf ("\nTesting pythonize library\n\n");
    testAppendToSysPath ();
    testImportModule ();
    testGetNewObjectRef ();
    testRunFunction ();
    testRunString ();
    testRunScript ();

    testDtor ();

    if (failed)
        printf ("\n*** %i tests failed ***\n\n", failed);
    else
        printf ("\nAll tests passed\n\n");
}

void TestPythonize::reportResult (const char *test, bool result)
{
    if (result)
        printf ("Test %-20s -- OK\n", test);
    else
    {
        printf ("Test %-20s -- *** Failed ***\n", test);
        failed++;
    }
}




TestPythonizeC::TestPythonizeC ()
{
    failed = 0;

    pyize  = initialize ();
    testInit = getPythonInit ();
    reportResult ("pythonInit", testInit);

    if (!testInit) return;

    setMainModule ();
    reportResult ("sysModule", getSysModule () ? true : false);
    reportResult ("mainModule", getMainModule ()  ? true : false);
 }

TestPythonizeC::~TestPythonizeC ()
{
}

  // bool appendToSysPath (const char* newPath)
bool TestPythonizeC::testAppendToSysPath ()
{
    reportResult ("appendToSysPath - 1", appendToSysPath (PATH_HERE));
    char *pyPath = Py_GetPath ();

    PyObject *pathList = getNewObjectRef (getSysModule (), "path");
    reportResult ("appendToSysPath - 2", pathList ? true : false);

    int len = PyList_GET_SIZE (pathList);

    // PyList_GetItem returns a borrowed reference
    char *lastItem = PyString_AsString (PyList_GetItem (pathList, len - 1));
    reportResult ("appendToSysPath - 3", strcmp (lastItem, PATH_HERE) == 0);

    decref (pathList);
}


  //PyObject *importModule (char *moduleName)
bool TestPythonizeC::testImportModule ()
{
    impModule = importModule ("testModule");
    reportResult ("importModule",  impModule ? true : false);
}


  //PyObject getNewObjectRef (PyObject *module, char *object) { return PyObject_GetAttrString (module, object) }
bool TestPythonizeC::testGetNewObjectRef ()
{
    pyAdd = getNewObjectRef (impModule, "add");
    reportResult ("getNewObjectRef", pyAdd ? true : false);

    decref (pyAdd);
}


  // bool runScript (char *scriptPath)
bool TestPythonizeC::testRunScript ()
{
    reportResult ("runScript", runScript ("helloc.py"));
}


  // bool runString (char *str)
bool TestPythonizeC::testRunString ()
{
    reportResult ("runString - 1", runString ("x = 5\n"));
    PyObject *pyX = getNewObjectRef (getMainModule (), "x");
    reportResult ("runString - 2", PyInt_AsLong (pyX) == 5);

    decref (pyX);
}


  // ReturnStruct runFunction (PyObject *object, PyObject *args)
bool TestPythonizeC::testRunFunction ()
{
    PyObject *addArgs = Py_BuildValue ("ii", 7, 6);
    PyObject *o   = runFunction (pyAdd, addArgs);
    reportResult ("runFunction - 1", o != NULL);
    reportResult ("runFunction - 2", PyInt_Check (o));

    long result = 0;
    if (o)
        result = PyInt_AS_LONG (o);

    reportResult ("runFunction - 3", result == 13);
    decref (addArgs);
    decref (o);
}

bool TestPythonizeC::testDtor ()
{
    printf ("\n\n");
    finalize ();
}

bool TestPythonizeC::runTests ()
{
    printf ("\nTesting pythonize C library\n\n");
    testAppendToSysPath ();
    testImportModule ();
    testGetNewObjectRef ();
    testRunFunction ();
    testRunString ();
    testRunScript ();

    testDtor ();

    if (failed)
        printf ("\n*** %i tests failed ***\n\n", failed);
    else
        printf ("\nAll tests passed\n\n");
}

void TestPythonizeC::reportResult (const char *test, bool result)
{
    if (result)
        printf ("Test %-20s -- OK\n", test);
    else
    {
        printf ("Test %-20s -- *** Failed ***\n", test);
        failed++;
    }
}


int main ()
{
    TestPythonize *tp;
    tp = new TestPythonize ();
    if (tp && tp->testInit)
        tp->runTests ();
    else
    {
        if (!tp)
            printf ("\nFailed to create TestPythonize\n\n");
        else
            if (!tp->testInit)
                printf ("\nPythonize constructor failed\n\n");
    }

    if (tp) delete tp;

    printf ("\nTesting C wrapper ...\n");

    TestPythonizeC *tpc;
    tpc = new TestPythonizeC ();
    if (tpc && tpc->testInit)
        tpc->runTests ();
    else
    {
        if (!tpc)
            printf ("\nFailed to create TestPythonizeC\n\n");
        else
            if (!tpc->testInit)
                printf ("\nPythonize constructor failed\n\n");
    }

    if (tpc) delete tpc;

}
