"""
Copyright 2003 Jim Bublitz

Terms and Conditions

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
COPYRIGHT HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

Except as contained in this notice, the name of the copyright holder shall
not be used in advertising or otherwise to promote the sale, use or other
dealings in this Software without prior written authorization from the
copyright holder.
"""

import sys, os, signal, time

from dcop import DCOPObject
from kdecore import KApplication, KCmdLineArgs, KAboutData, KStandardDirs, KGlobal, KProcIO, KProcess,\
                    KURL, NETRootInfo, KAboutData, KAboutPerson, KCompletion, KEntry, KEntryKey, testKEntryMap
from kdeui import KMainWindow, KCommandHistory, KDateValidator, KEdit, KEditListBox, KPasswordDialog,\
                  KStringListValidator, KMimeTypeValidator, KStdAction, KLineEdit, testKActionList
from kio import KIO, KMimeType, testKIOUDSEntry, testKIOUDSEntryList, testKIOMetaData
from kfile import KFile #, KNotifyDialog
from kparts import KParts, testQMapQCStringInt


from qt import QString, QPushButton, SIGNAL, Qt, QDir, QStringList, QCheckBox, QCString

False = 0
True  = 1

class MainWin (KMainWindow):
    def __init__ (self, *args):
        apply (KMainWindow.__init__, (self,) + args)
        self.setGeometry (10, 10, 400, 400)

        self.edit = KEdit (self)
        self.edit.setGeometry (10, 10, 220, 150)

        textList = ["Now is the winter of our discontent",\
                    "made glorious summer by this sun of York;",\
                    "and all the clouds that lour'd upon our house",\
                    "in the deep bosom of the ocean buried."]

        for line in textList:
            self.edit.insertLine (line)

        self.lineEdit = KLineEdit (self)
        self.lineEdit.setGeometry (240,10, 50, 20)

        self.editListBox = KEditListBox (self)
        self.editListBox.setGeometry (10, 180, 220, 200)

        self.locCB = QCheckBox ("Locale", self)
        self.locCB.setGeometry (290, 300, 100, 20)
        self.locCB.setChecked (True)
        self.dlgCB = QCheckBox ("Dialogs", self)
        self.dlgCB.setGeometry (290, 325, 100, 20)
#        self.dlgCB.setChecked (True)

        self.testBtn = QPushButton ("test", self)
        self.testBtn.setGeometry (290, 355, 60, 22)
        self.connect (self.testBtn, SIGNAL ("clicked ()"), self.doTest)

        self.newAction    = KStdAction.openNew (self.noSlot, self.actionCollection (), "openNew")
        self.openAction   = KStdAction.open (self.noSlot, self.actionCollection (), "open")
        self.saveAction   = KStdAction.save (self.noSlot, self.actionCollection (), "save")
        self.saveAsAction = KStdAction.saveAs (self.noSlot, self.actionCollection (), "saveAs")
        self.printAction  = KStdAction.print_ (self.noSlot, self.actionCollection (), "print")
        self.quitAction   = KStdAction.quit (self.noSlot, self.actionCollection (), "quit")


    def noSlot (self):
        pass

    def doTest (self):
        test = TestULongLong ()
        test = TestDCOPClient ()
        test = TestDCOPObject ()
        test = TestKApplication ()
        test = TestKCharsets ()
        if not self.locCB.isChecked ():
            test = TestKDateValidator ()
        test = TestKMimeTypeValidator ()
        test = TestKStringListValidator ()
        test = TestKEdit ()
        test = TestKEditListBox ()
        test = TestKFile ()
        if self.locCB.isChecked ():
            test = TestKLocale ()
        test = TestKMimeType ()
        test = TestKProcIO ()
        test = TestKURLOps ()
        test = TestKURLList ()
        test = TestNETRootInfo ()
        test = TestKIOSlave ()
        test = TestKIONetRCAutoLogin ()
        test = TestKPartsBrowserExtension ()
        test = TestKIOglobal ()
#XXX not working with 3.6/works with 3.5        test = TestKAboutData ()
        test = TestKActionCollection ()
        test = TestKActionList ()
        test = TestKCompletionBase ()
        test = TestKEntryMap ()
        if self.dlgCB.isChecked ():
#            test = TestKNotifyDialog ()
            test = TestKPasswordDialog ()
            test = TestKIOPasswordDialog ()

        sys.exit (0)

#------------------ ulonglong ---------------------------

class TestULongLong:
    def __init__ (self):
        print "\n\nTesting ulonglong ...\n"
        print "KIO.convertSize (0)",  KIO.convertSize (0)
        print "KIO.number (0)", KIO.number (0)
        print "KIO.convertSize (1)",  KIO.convertSize (1)
        print "KIO.number (1)", KIO.number (1)
        print "KIO.convertSize (1000)",  KIO.convertSize (1000)
        print "KIO.number (1000)", KIO.number (1000)
        print "KIO.convertSize (2048)",  KIO.convertSize (2048)
        print "KIO.number (2048)", KIO.number (2048)
        print "KIO.convertSize (1234567890123456789)",  KIO.convertSize (1234567890123456789)
        print "KIO.number (1234567890123456789)", KIO.number (1234567890123456789)
        print


#------------------ DCOPClient -------------------------

class TestDCOPClient:
    def __init__ (self):
        print "Testing DCOPClient ...\n"
        print "attached =", KApplication.dcopClient ().isRegistered ()
        dcop = KApplication.dcopClient ()
        print "registeredApps",
        regApps = dcop.registeredApplications ()
        for application in regApps:
            print application,
        print
        print "remoteObjects"
        remObjs, ok = dcop.remoteObjects ("klauncher")
        remIfs = []
        if not ok:
            print "remoteObjects returned not OK"
        else:
            for obj in remObjs:
                print obj
                ifs = dcop.remoteInterfaces ("klauncher", obj)[0]
                for i in ifs:
                    print "    If :", i
                fcns = dcop.remoteFunctions ("klauncher", obj)[0]
                for f in fcns:
                    print "    Fcn:", f
            print

#------------------ DCOPObject -------------------------

class TestDCOPObject:
    def __init__ (self):
        print "Testing DCOPObject ...\n"
        print "match:", DCOPObject.match ("K")


#----------------- KApplication -------------------------

class TestKApplication:
    def __init__ (self):
        print "\nTesting KAppplication ...\n"
        print "startServiceByName"
        result, error, dcopService, pid = KApplication.startServiceByName ("KWrite", QString.null, "kwrite")
        print "result     :", result
        print "error      :", error
        print "dcopService:", dcopService
        print "pid        :", pid

        if pid: os.kill (pid, signal.SIGKILL)
        print

        print "startServiceByDesktopName"
        result, error, dcopService, pid = KApplication.startServiceByDesktopName ("kwrite", QString.null, "kwrite")
        print "result     :", result
        print "error      :", error
        print "dcopService:", dcopService
        print "pid        :", pid

        if pid: os.kill (pid, signal.SIGKILL)
        print

        stddir = KStandardDirs ()
        kwPath = stddir.findResource ("apps", "Editors/kwrite.desktop")
        print "startServiceByDesktopPath"
        result, error, dcopService, pid = KApplication.startServiceByDesktopPath (kwPath, QString.null, "kwrite")
        print "result     :", result
        print "error      :", error
        print "dcopService:", dcopService
        print "pid        :", pid

        if pid: os.kill (pid, signal.SIGKILL)
        print

        print "kdeinitExec"
        result, error, pid = KApplication.kdeinitExec ('kwrite')
        print "result     :", result
        print "error      :", error
        print "pid        :", pid

        if pid: os.kill (pid, signal.SIGKILL)
        print


#----------------- KCharsets -------------------------

class TestKCharsets:
    def __init__ (self):
        print "Testing KCharsets ...\n"
        cs = KGlobal.charsets ()
        tc, ok = cs.codecForName2 ("Big5")
        print tc, ok, tc.name ()

        print
        print "undoLimit, redoLimit"
        ch = KCommandHistory ()
        print ch.undoLimit (), ch.redoLimit ()
        ch.setUndoLimit (30)
        ch.setRedoLimit (10)
        print ch.undoLimit (), ch.redoLimit ()
        print

#----------------- KDateValidator -------------------------

class TestKDateValidator:
    def __init__ (self):
        print "Testing KDateValidator ...\n"
        dv = KDateValidator ()
        print dv.validate ("1-4-2003")
        print dv.validate ("33-04-03")
        print dv.validate ("3-7")
        print dv.validate ("3-")
        print

#----------------- KMimeTypeValidator -------------------------

class TestKMimeTypeValidator:
    def __init__ (self):
        print "Testing KMimeTypeValidator ...\n"
        mv = KMimeTypeValidator (mainWindow)
        print mv.validate ("text/plain")
        print mv.validate ("/test/plain")
        print

#----------------- KStringListValidator -------------------------

class TestKStringListValidator:
    def __init__ (self):
        print "Testing KStringListValidator ...\n"
        sv = KStringListValidator ()
        sl = QStringList ()
        sl.append ("Hello")
        sl.append ("Goodbye")
        sv.setStringList (sl)

        sv.setRejecting (False)
        print sv.validate ("Hello")
        print sv.validate ("Good")
        print sv.validate ("Something")

        sv.setRejecting (True)
        print sv.validate ("Hello")
        print sv.validate ("Good")
        print sv.validate ("Something")
        print

#----------------- KEdit -------------------------

class TestKEdit:
    def __init__ (self):
        print "Testing KEdit ...\n"
        print mainWindow.edit.posToRowCol (50)
        print

#----------------- KEditListBox -------------------------

class TestKEditListBox:
    def __init__ (self):
        print "Testing KEditListBox ...\n"
        itemList = ["one", "two", "three", "four"]
        mainWindow.editListBox.insertStrList (itemList, -1)
        retrList = mainWindow.editListBox.items ()
        for i in range (len (itemList)):
            print retrList [i], itemList [i]
        print

#----------------- KFile -------------------------

class TestKFile:
    def __init__ (self):
        print "Testing KFile ...needs repair !!!\n"
        return
        print "KFile.isSortByName", KFile.isSortByName (QDir.Name)
        print "KFile.isSortBySize", KFile.isSortBySize (QDir.Size)
        print "KFile.isSortByDate", KFile.isSortByDate (QDir.Time)
        print "KFile.isSortDirsFirst", KFile.isSortDirsFirst (QDir.DirsFirst)
        print "KFile.isSortCaseInsensitive", KFile.isSortCaseInsensitive (QDir.IgnoreCase)
        print "KFile.isDefaultView", KFile.isDefaultView (KFile.Default)
        print "KFile.isSimpleView", KFile.isSimpleView (KFile.Simple)
        print "KFile.isDetailView", KFile.isDetailView (KFile.Detail)
        print "KFile.isSeparateDirs", KFile.isSeparateDirs (KFile.SeparateDirs)
        print "KFile.isPreviewContents", KFile.isPreviewContents (KFile.PreviewContents)
        print

#----------------- KLocale -------------------------

class TestKLocale:
    def __init__ (self):
        print "Testing KLocale ...\n"
        locale = KGlobal.locale ()
        print locale.readMoney ("$12.34")
        print locale.readNumber ("1,234,567")
        qd, ok = locale.readDate (QString ("2003-10-10"))
        print qd, qd.isValid ()
        print ok, qd.day (), qd.month (), qd.year ()
        qtm, ok = locale.readTime ("11:12:13")
        print ok, qtm.hour (), qtm.minute (), qtm.second ()
        print

#----------------- KMimeType -------------------------

class TestKMimeType:
    def __init__ (self):
        print "Testing KMimeType ...\n"
        m    = open ('manualtest.py', 'r')
        buff = m.read ()
        m.close ()
        ptr, acc = KMimeType.findByContent (buff)
        print acc, ptr.name ()
        ptr, acc = KMimeType.findByFileContent ('manualtest.py')
        print acc, ptr.name ()

#----------------- KProcIO -------------------------

class TestKProcIO:
    def __init__ (self):

        print "\nTesting KProcIO ...\n"
        p = KProcIO ()
        p.setExecutable ("python")
        p.setArguments (["testKProcIO.py"])
        p.connect (p, SIGNAL ("readReady (KProcIO *)"), self.procRead)
        p.enableReadSignals (1)
        p.start (KProcess.Block)

    def procRead (self, p):
        print "caught readReady"
        result, buff, partial = p.readln ()
        print result, buff, partial
        if result != -1:
            os.kill (p.pid (), signal.SIGKILL)

#----------------- KURL.Ops -------------------------
class TestKURLOps:
    def __init__ (self):

        print "\nTesting KURL.Operators ...\n"
        url1 = "http://riverbankcomputing.co.uk"
        url2 = "http://python.org"

        kurl1  = KURL (url1)
        kurl2  = KURL (url2)
        kurl3  = KURL (url1)
        qs1    = QString (url1)
        qs2    = QString (url2)

        # bool operator==( const KURL& _u ) const
        if kurl1 == kurl3:
            print "== (KURL) OK"
        else:
            print "== (KURL) *** Fails"

        # bool operator==( const QString& _u ) const
        if kurl1 == qs1:
            print "== QString OK"
        else:
            print "== QString *** Fails"
            print "QString:", qs1
            print "KURL:   ", kurl1
            print

        # bool operator!=( const KURL& _u ) const
        if kurl1 != kurl2:
            print "!= KURL OK"
        else:
            print "!= KURL *** Fails"

        # bool operator!=( const QString& _u ) const
        if kurl2 != qs1 and not kurl1 != qs1:
            print "!= QString OK"
        else:
            print "!= QString *** Fails"



#----------------- KURL.List -------------------------

class TestKURLList:
    def __init__ (self):

        print "\nTesting KURL.List ...\n"
        k = KURL.List ()
        k.append (KURL ("http://slashdot.org"))
        k.append (KURL ("http://kde.org"))
        k.append (KURL ("yahoo.com"))

        for i in range (len (k)):
            print k [i].url ()

        print KURL("yahoo.com") in k
        print KURL('riverbankcomputing.co.uk') in k

#----------------- NETRootInfo -------------------------

class TestNETRootInfo:
    def __init__ (self):
        print "\nTesting NETRootInfo ...\n"
        nr = NETRootInfo (0xfff)
        print nr.wmName ()
        print nr.screenNumber ()
        print nr.supported ()
        print nr.clientList ()
        print nr.clientListStacking ()
        print nr.kdeSystemTrayWindows ()
        print nr.virtualRoots ()


#----------------- KIO.Slave -------------------------

class TestKIOSlave:
    def __init__ (self):
        print "\nTesting KIO.Slave ...\n"
        slave, err, errText = KIO.Slave.createSlave ("http", KURL("http://www.kde.org"))
        print slave, err, errText
        print slave.protocol ()
        slave.kill ()


#----------------- KIO.NetRC.AutoLogin -------------------------

class TestKIONetRCAutoLogin:
    def __init__ (self):
        print "\nTesting KIO.NetRC.AutoLogin ...\n"
        strings  = ["one", "two", "three"]
        qstrings = []
        d  = {}
        for s0 in strings:
            slist = QStringList ()
            for s in strings:
                slist.append (s0 + " " + s)

            qstrings .append (QString (s0))
            d [qstrings [-1]] = slist

        al = KIO.NetRC.AutoLogin ()
        al.type     = "type"
        al.machine  = "machine"
        al.login    = "login"
        al.password = "password"
        al.macdef   = d

        print al.type
        print al.machine
        print al.login
        print al.password
        d = al.macdef
        for key in d.keys ():
            print str (key)
            for i in range (d [key].count ()):
                print "    ", str (d [key][i])


#----------------- KParts.BrowserExtension -------------------------

class TestKPartsBrowserExtension:
    def __init__ (self):
        print "\nTesting KParts.BrowserExtension ...\n"
        d = KParts.BrowserExtension.actionSlotMap ()
        for item in d.keys ():
            print item, d [item]
        print

        nlist = [QCString ("zero"), QCString ("one"), QCString ("two"), QCString ("three")]
        dict  = {}
        for i in range (len (nlist)):
            dict [nlist [i]] = i

        retDict = testQMapQCStringInt (dict)
        for key in retDict.keys ():
            print key, retDict [key]


#----------------- KIO.global -------------------------

class TestKIOglobal:
    def __init__ (self):
        print "\nTesting KIO.global ...\n"
        nlist        = [QString ("zero"), QString ("one"), QString ("two"), QString ("three")]
        d            = {}
        udsentry     = []
        udsentrylist = []

        for i in range (len (nlist)):
            d [nlist [i]] = nlist [i].upper ()
            atom          = KIO.UDSAtom ()
            atom.m_str    = nlist [i]
            atom.m_long   = -i
            atom.m_uds    = i

            udsentry.append (atom)
            udsentrylist.append (list (udsentry))

        retD = testKIOMetaData (d)
        for key in retD.keys ():
            print key, retD [key]
        print

        retUDSEntry = testKIOUDSEntry (udsentry)
        for item in retUDSEntry:
            print item.m_str, item.m_long, item.m_uds
        print

        retUDSEntryList = testKIOUDSEntryList (udsentrylist)
        for ulist in retUDSEntryList:
            for item in ulist:
                print item.m_str, item.m_long, item.m_uds
            print


#----------------- KAboutData -------------------------


class TestKAboutData:
    def __init__ (self):
        print "\nTesting KAboutData ...\n"
        ad = KAboutData ("test", "manualtest", "1.0")
        t = []
        e = []
        for i in range (4):
            task  = "".join (["task", str(i)])
            email = "".join (["email", str(i)])
            web   = "".join (["web", str(i)])
            ad.addAuthor ("".join (["author", str(i)]), task, email, web)
            ad.addAuthor ("author" + str (i))
            t.append ("".join (["translator", str(i)]))
            e.append (email)

        ap = KAboutPerson ("author", task, email, web)
        ad.setTranslator (",".join (t), ",".join (e))

        authors = ad.authors ()
        translators = ad.translators ()

        for i in range (4):
            print str (authors [i].name ()), str (authors [i].task ()), authors [i].emailAddress (), authors [i].webAddress ()
        print

        for i in range (len (translators)):
            print translators [i].name (), translators [i].emailAddress ()



#----------------- KActionCollection -------------------------

class TestKActionCollection:
    def __init__ (self):
        print "\nTesting KActionCollection ...\n"
        ac = mainWindow.actionCollection ()
        print ac.actions ()
        for action in ac.actions ():
            print action.name ()


#----------------- KActionList -------------------------

class TestKActionList:
    def __init__ (self):
        print "\nTesting KActionList ...\n"
        actionList = testKActionList (mainWindow.actionCollection ().actions ())
        for action in actionList:
            print action.name ()

#----------------- KCompletionBase -------------------------

class TestKCompletionBase:
    def __init__ (self):
        print "\nTesting KCompletionBase ...\n"
        kcomp = KCompletion ()
        mainWindow.lineEdit.setCompletionObject (kcomp)
        keymap = mainWindow.lineEdit.getKeyBindings ()
        print keymap


#----------------- KEntryMap -------------------------

class TestKEntryMap:
    def __init__ (self):
        print "\nTesting KEntryMap ...\n"


        ke1 = KEntry ()
        ke1.mValue     = "value1"
        ke1.bDirty     = 0
        ke1.bNLS       = 0
        ke1.bGlobal    = 0
        ke1.bImmutable = 0
        ke1.bDeleted   = 0

        ke2 = KEntry ()
        ke2.mValue     = "value2"
        ke2.bDirty     = 1
        ke2.bNLS       = 1
        ke2.bGlobal    = 1
        ke2.bImmutable = 1
        ke2.bDeleted   = 1


    #   bExpand KDE >= 3.1.0

        kek1 = KEntryKey ()
        kek1.mGroup   = "group1"
        kek1.mKey     = "key1"
        kek1.bLocal   = 1
        kek1.bDefault = 1
        kek1.c_key    = "c_key1"


        kek2 = KEntryKey ()
        kek2.mGroup   = "group2"
        kek2.mKey     = "key2"
        kek2.bLocal   = 0
        kek2.bDefault = 0
        kek2.c_key    = "c_key2"

        kdictIn = {}
        kdictIn [kek1] = ke1
        kdictIn [kek2] = ke2

        kdictOut = testKEntryMap (kdictIn)

        for keKey in kdictOut.keys ():
            print keKey.mGroup, keKey.mKey
            print "    ", keKey.bLocal, keKey.bDefault, keKey.c_key
            k = kdictOut [keKey]
            print "    ",     k.mValue, k.bDirty, k.bNLS, k.bGlobal, k.bImmutable, k.bDeleted
            print




#----------------- KNotifyDialog -------------------------

#class TestKNotifyDialog:
#    def __init__ (self):
#print "Testing KNotifyDialog ...\n"

#result = KNotifyDialog.configure (self)
#print "exec result =",result

#----------------- KPasswordDialog -------------------------

class TestKPasswordDialog:
    def __init__ (self):
        print "\nTesting KPasswordDialog ...\n"
        result, passwd, keep = KPasswordDialog.getPassword ("Enter Password")
        print result, passwd, keep

        result, passwd, keep = KPasswordDialog.getPassword ("Enter Password", 1)
        print result, passwd, keep


#----------------- KIO.PasswordDialog -------------------------

class TestKIOPasswordDialog:
    def __init__ (self):
        print "\nTesting KIO.PasswordDialog ...\n"
        result, user, password, keep = KIO.PasswordDialog.getNameAndPassword (0)
        print result, user, password, keep

        result, user, password, keep = KIO.PasswordDialog.getNameAndPassword (1)
        print result, user, password, keep





#-------------------- main ------------------------------------------------

description = "A basic application template"
version     = "1.0"
aboutData   = KAboutData ("", "",\
    version, description, KAboutData.License_GPL,\
    "(C) 2003 whoever the author is")

aboutData.addAuthor ("author1", "whatever they did", "email@somedomain")
aboutData.addAuthor ("author2", "they did something else", "another@email.address")

KCmdLineArgs.init (sys.argv, aboutData)

KCmdLineArgs.addCmdLineOptions ([("+files", "File to open")])

app = KApplication ()

mainWindow = MainWin (None, "main window")

mainWindow.show()
app.exec_loop()

